from math import *
from longfloat import longfloat

class Quadrature:
    def __init__(self, order, region=[-1.,1.], rule='GaussLegendre',
                 uselongfloat=0):
        self.uselongfloat = uselongfloat

        if uselongfloat:
            region = [longfloat(region[0]),longfloat(region[1])]
            
        self.order = order
        self.region = region
        self.midpoint = (region[1]+region[0])/2.
        self.scale = (region[1]-region[0])/2.
        self.rule = rule

        if rule == 'GaussLegendre':
            x,w = self.__grule()
            for i in range(self.order):
                x[i] = x[i]*self.scale + self.midpoint
                w[i] = w[i]*self.scale
        elif rule == 'GaussHermite':
            raise "sorry this is not tested"
            x,w = self.__hrule()
        elif rule == 'GaussLaguerre':
            raise "sorry this is not tested"
            x,w = self.__lrule()
        else:
            raise "unknown rule",rule

        self.x = x   
        self.w = w

    def __pn(self, n,x):
        '''
        Evaluate the Legendre polynomials up to the given order(n)
        at x in [-1,1]
        '''
        p = range(n+1)
        p[0] = 1
        if n == 0:
            return p
        p[1] = x
        for i in range(1,n):
            p[i+1] = i*(x*p[i] - p[i-1])/(i+1) + x*p[i]
        return p

    def __hn(self, n, x):
        '''
        Evaluate the renormalized Hermite polynomials Hn(x).

        Regular Hn
        p[0] = 1
        p[1] = 2*x
        p[i+1] = 2*x*p[i] - 2*i*p[i-1]

        Renormalized
        p[0] = 1/pi**(1/4)
        p[1] = sqrt(2)*x*p[0]
        p[i+1] = x*sqrt(2/(i+1))*p[i] - sqrt(j/(j+1))*p[j-1]
        
        '''
        p = range(n+1)
        p[0] = 1.0
        if self.uselongfloat:
            x = longfloat(x)
            p[0] = longfloat(1)
        if n == 0:
            return p
        p[1] = 2*x
        for i in range(1,n):
            p[i+1] = 2*x*p[i] - 2*i*p[i-1]
        return p

    def __ln(self, n, x):
        '''
        Laguerre polyn
        '''
        p = range(n+1)
        p[0] = 1.0
        if self.uselongfloat:
            x = longfloat(x)
            p[0] = longfloat(1)
        if n == 0:
            return p
        p[1] = 1-x
        for i in range(1,n):
            p[i+1] = ((2*i+1-x)*p[i]-i*p[i-1])/(i+1)
        return p
        

    def __grule(self):
        ''' Return the Gauss Legendre quadrature weights. '''

        n = self.order

        nbits = 52
        if self.uselongfloat:
            nbits = longfloat.nbits
        acc = 0.5**(nbits/3 + 10)
        if self.uselongfloat:
            acc = longfloat(acc)

        # References made to the equation numbers in Davis & Rabinowitz 2nd ed.
        roots   = range(n)
        weights = range(n)
        for k in range(n):
            # Initial guess for the root using 2.7.3.3b
            #x = (1.0 - 1.0/(8*n*n) + 1.0/(8*n*n*n))*cos(((4*k+3.0)/(4*n+2))*pi)
            x = (1.0 - 1.0/(8.0*n*n) + 1.0/(8.0*n*n*n))*cos(((4*k+3.0)/(4*n+2))*pi)
            if self.uselongfloat:
                x = longfloat(x)
            # Refine the initial guess using a 3rd-order Newton correction 2.7.3.9
            # Running this six times will ensure enough precision even if we're
            # requesting over 1500 significant decimal figures.
            for iter in range(7):
                p = self.__pn(n,x)
                p0 = p[n]                         # Value
                p1 = n*(p[n-1] - x*p[n])/(1-x*x)  # First derivative ... 2.7.3.5
                p2 = (2*x*p1 - n*(n+1)*p0)/(1-x*x)# Second derivative
                delta = - (p0/p1)*(1 + (p0*p2)/(2*p1*p1))
                x = x + delta
                if abs(delta) < acc:
                    break
            if iter >= 6:
                raise "netwon iteration for root failed"

            # Compute the weight using 2.7.3.8
            p = self.__pn(n,x)
            w = 2*(1-x*x)/(n*n*p[n-1]*p[n-1])

            roots[k] = x
            weights[k] = w

        return (roots,weights)

    def __factorial(self,n):
        f = 1.0
        if self.uselongfloat:
            f = longfloat(1.0)
        for i in range(1,n+1):
            f = f*i
        return f

    def __hrule(self):
        '''

        Gauss-Hermite rule on [0,inf]

        '''
        n = self.order

        nbits = 52
        rootpi = sqrt(pi)
        two = 2.0
        zero = 0.0
        if self.uselongfloat:
            two = longfloat(2)
            zero = longfloat(0)
            rootpi = longfloat(1).pi().sqrt()
            nbits = longfloat.nbits
        acc = 0.5**(nbits/3 + 10)
        if self.uselongfloat:
            acc = longfloat(acc)

        # Find the points which are the zeros of Hn
        # dH[n]/dx = 2*n*H[n-1]

        npt = (n-1)/2+1
        roots = range(npt)
        weights = range(npt)
        nfound = 0
        if n%2:
            p = self.__hn(n+1,0.0)
            roots[0] = zero
            weights[0] = two**(n+1)*self.__factorial(n)*rootpi/p[n+1]**2
            # !!!! Since just doing [0,inf] need to halve this
            weights[0] = weights[0]/two
            nfound = nfound + 1

        guess = 0.01
        if self.uselongfloat:
            guess = longfloat(guess)
        while nfound != npt:

            # Stupid linear search forward for sign change
            x = guess
            p = self.__hn(n,x)
            hh = p[n]
            while 1:
                x = x + 0.05
                p = self.__hn(n,x)
                if hh*p[n] < 0.0:
                    break
                
            # Cubic newton
            for iter in range(7):
                p = self.__hn(n,x)
                p0 = p[n]               # Value
                p1 = 2*n*p[n-1]         # First derivative
                p2 = 4*n*(n-1)*p[n-2]   # Second derivative
                delta = - (p0/p1)*(1 + (p0*p2)/(2*p1*p1))
                #print "newton", x, delta
                x = x + delta
                if abs(delta) < acc:
                    break
            if iter >= 6:
                raise "Hermite netwon iteration for root failed"

            new = 1
            for i in range(nfound):
                if abs(x-roots[i]) < 1e-3:
                    new = 0
                    break
            if not new:
                guess = guess + 0.1
                if guess > 100.0:
                    raise "Hermite root finder is lost"
                continue

            #print "found root", x
            roots[nfound] = x
            p = self.__hn(n+1,x)
            weights[nfound] = two**(n+1)*self.__factorial(n)*rootpi/p[n+1]**2
            nfound = nfound + 1

            guess = x + 0.05

        return roots, weights

    def __lrule(self):
        '''

        Gauss-Laguerre rule on [0,inf]

        '''
        n = self.order

        nbits = 52
        zero = 0.0
        if self.uselongfloat:
            zero = longfloat(0)
            nbits = longfloat.nbits
        acc = 0.5**(nbits/3 + 10)
        if self.uselongfloat:
            acc = longfloat(acc)

        # Find the points which are the zeros of Ln
        # dL[n]/dx   =  n*(L[n] - L[n-1])/x

        roots = range(n)
        weights = range(n)
        nfound = 0
        guess = 0.0001
        step = 0.0001
        if self.uselongfloat:
            guess = longfloat(guess)
        while nfound != n:
            # Stupid linear search forward for sign change
            x = guess
            p = self.__ln(n,x)
            hh = p[n]
            if nfound > 1:
                step = 0.1*(roots[nfound-1]-roots[nfound-2])
            print "step", step
            niter = 1000
            while niter:
                niter = niter - 1
                xp = x
                x = x + step
                p = self.__ln(n,x)
                if hh*p[n] < 0.0:
                    break
            x = (x + xp)/2.0
            if niter == 0:
                print nfound, x
                raise "Gauss Laguerre lost a root?"

            print "linear search found", x, hh, p[n]
                
            # Cubic newton
            for iter in range(7):
                p = self.__ln(n,x)
                p0 = p[n]               # Value
                p1 = n*(p[n] - p[n-1])/x # First derivative
                pp1 = (n-1)*(p[n-1] - p[n-2])/x
                p2 = n*(p1 - pp1)/x - p1/x    # Second derivative
                delta = - (p0/p1)*(1 + (p0*p2)/(2*p1*p1))
                x = x + delta
                if abs(delta) < acc:
                    break
            if iter >= 6:
                raise "Laguerre netwon iteration for root failed"
            print "found",x

            new = 1
            for i in range(nfound):
                if abs(x-roots[i]) < 1e-3:
                    new = 0
                    break
            if not new:
                guess = guess + step
                if guess > 150.0:
                    raise "Laguerre root finder is lost"
                continue

            p = self.__ln(n+1,x)
            roots[nfound] = x
            weights[nfound] = x/((n+1)*p[n+1])**2
                
            nfound = nfound + 1

            guess = x + 0.05

        return roots, weights

    def integrate(self, function=None, values=None):
        n = len(self.x)
        if function:
            values = range(n)
            for i in range(n):
                values[i] = function(self.x[i])
        sum = 0.0
        for i in range(n):
            sum = sum + self.w[i]*values[i]
        return sum

    def points(self):
        return self.x

    def weights(self):
        return self.w

def QuadratureTest(uselongfloat=0):
    ''' Test the Gauss-Legendre quadrature on [0.5,1] '''
    maxmaxerr = 0.0
    for order in range(1,20):
        q = Quadrature(order,[0.5,1.0],uselongfloat=uselongfloat)
        maxerr = 0.0
        for power in range(2*order):
            fstr = 'def __qtestf(x): return x**%d' % power
            exec(fstr)
            value = q.integrate(__qtestf)
            del(__qtestf)
            if uselongfloat:
                test = (longfloat(1) - longfloat((-power-1,1)))/(power+1.0)
            else:
                test = (1.0 - 0.5**(power+1))/(power+1.0)
            maxerr = max(maxerr,abs(float(value-test)))
        print "order %d:  integrates powers up to %d with maxerr=%e" % \
              (order, 2*order-1, maxerr)
        maxmaxerr = max(maxmaxerr,maxerr)
    print (' Maximum error from all orders %9.1e' % float(maxmaxerr))

def QuadratureTest2(uselongfloat=0):
    ''' Test the Gauss-Hermite quadrature on [-inf..inf] '''
    maxmaxerr = 0.0
    rootpi = sqrt(pi)
    two = 2.0
    if uselongfloat:
        two = longfloat(2)
        rootpi = two.pi().sqrt()

    for order in range(2,24):
        q = Quadrature(order,rule="GaussHermite",
                       uselongfloat=uselongfloat)
        maxerr = 0.0
        for power in range(0,2*order,2):
            fstr = 'def __qtestf(x): return x**%d' % power
            exec(fstr)
            value = q.integrate(__qtestf)
            del(__qtestf)
            test = 0.5*rootpi*0.5**(power/2)
            for i in range(power-1,0,-2):
                test = test * i
            maxerr = max(maxerr,abs(float(value-test)/test))
        print "order %d:  integrates powers up to %d with maxerr=%e" % \
              (order, 2*order-2, maxerr)
        maxmaxerr = max(maxmaxerr,maxerr)
    print (' Maximum error from all orders %9.1e' % float(maxmaxerr))


if __name__ == '__main__':

    q = Quadrature(7,(0.0,1.0),rule='GaussLegendre')
    print q.points()
    print q.weights()
    stop

    print ' \n\n Testing quadrature with default floats \n'
    QuadratureTest()

    print ' \n\n Testing quadrature with long floats \n'
    QuadratureTest(uselongfloat=1)
    
    #QuadratureTest2()

    #QuadratureTest2(1)



import sys, string

data = sys.stdin
basis = "STO-3G"

def find_string(s):
    n = len(s)
    while 1:
        line = data.readline()
        if not line:
            sys.exit(0)

        if line[0:n] == s:
            return line

def read_geometry():
    find_string("            XYZ format geometry")

    data.readline()
    data.readline()
    data.readline()
    atom = data.readline().split()[0]
    return atom

def read_numao():
    line = find_string("          AO basis - number of functions:").split()
    n = int(line[6])
    return n

def read_energy():
    find_string("     <<<<<<              called scfcvg")
    line = find_string("         Total DFT energy =").split()
    energy = float(line[4])
    return energy

def read_occn(n, ab):
    occ = [0]*n
    eps = [0]*n
    test = "                    DFT Final %s Molecular Orbital Analysis" % ab
    if ab == "Beta":
        test = " " + test
    find_string(test)
    
    for i in range(n):
        line = find_string(" Vector ").split()
        occ[i] = float(string.replace(line[2].split("=")[1],"D","e"))
        e = line[4]
        if e[0] == "S":
            e = line[3].split("=")[1]
        eps[i] = float(string.replace(e,"D","e"))
        
    return (occ,eps)

def matrix(n,m):
    V = [0]*n
    for i in range(n):
        V[i] = [0]*m
    return V

def read_matrix(n,m):
    V = matrix(n,m)
    for jlo in range(0,m,6):
        jhi = min(m,jlo+6)
        data.readline() # blank
        data.readline() # numbers
        data.readline() # underscore
        for i in range(n):
            line = data.readline().split()
            for jj in range(jhi-jlo):
                j = jlo + jj
                V[i][j] = float(line[1+jj])
    return V

def read_orbitals(n):
    find_string(" global array:")
    V = read_matrix(n,n)
    return V

def make_density(n, aocc, amo, bocc, bmo):
    D = matrix(n,n)
    for i in range(n):
        for j in range(n):
            sum = 0.0
            for k in range(n):
                sum = sum + aocc[k]*amo[i][k]*amo[j][k] + bocc[k]*bmo[i][k]*bmo[j][k]
            D[i][j] = sum
    return D

def print_matrix(n, m, a):
    for i in range(n):
        for j in range(m):
            print "%12.5f" % a[i][j],
        print ""

def print_vector(n, a):
    for i in range(n):
        print "%12.5f" % a[i],
    print ""

while 1:
    atom = read_geometry()
    n =  read_numao()
    e = read_energy()

    aocc, aeps = read_occn(n, 'Alpha')
    bocc, beps = read_occn(n, 'Beta')

    amo = read_orbitals(n)
    bmo = read_orbitals(n)

    density = make_density(n, aocc, amo, bocc, bmo)

    print "<atomicguess symbol=\"%s\" basis=\"%s\">" % (atom,basis)
    print "   <guessdensitymatrix>"
    print_matrix(n, n, density)
    print "   </guessdensitymatrix>"
    print "   <alphaocc>"
    print_vector(n, aocc)
    print "   </alphaocc>"
    print "   <betaocc>"
    print_vector(n, bocc)
    print "   </betaocc>"
    print "   <alphaeps>"
    print_vector(n, aeps)
    print "   </alphaeps>"
    print "   <betaeps>"
    print_vector(n, beps)
    print "   </betaeps>"
    print "   <alphavectors>"
    print_matrix(n, n, amo)
    print "   </alphavectors>"
    print "   <betavectors>"
    print_matrix(n, n, bmo)
    print "   </betavectors>"
    print "</atomicguess>"
    



    




#!/usr/bin/env python
# Write the name of the data file in the argument list
# Known issues: multiple file arguments get included 
# too many times
import sys, string, re, array, os
try:
        sys.argv[1]
except:
        print "Enter the file as an argument"
        sys.exit()
number  = []
time    = []
field   = []
energy  = []
norm    = []
overlap = []
xdip    = []
ydip    = []
zdip    = []
accel   = []
Wtime   = []
fileList = sys.argv[1:]

for file in fileList:
        print file
        f = open(file)
        lines = f.readlines()
        #Find the starting pattern
        while lines:
                line = lines.pop(0)
                m = re.match( "\s*step",line)
                if m:
                        break
        #get the data
        for line in lines:
                if re.match("\A\s*Active",line):
                        print "Found the End"
                        break
                if re.match("\A\s+\d+",line):
                        word = line.split()
                        number.append( word[-1])
                        time.append(   word[1])
                        field.append(  word[2])
                        energy.append( word[3])
                        norm.append(   word[4])
                        overlap.append(word[5])
                        xdip.append(   word[6])
                        ydip.append(   word[7])
                        zdip.append(   word[8])
                        accel.append(  word[9])
                        Wtime.append(  word[10])
                if not line:
                        break
timeFile   = open("time.dat",      'w')
energyFile = open("energy.dat",    'w')
fieldFile  = open("field.dat",     'w')
normFile   = open("norm.dat",      'w')
overlapFile= open("overlap.dat",   'w')
xdipFile   = open("xdip.dat",      'w')
ydipFile   = open("ydip.dat",      'w')
zdipFile   = open("zdip.dat",      'w')
accelFile  = open("accel.dat",     'w')
WtimeFile  = open("Wtime.dat",     'w')
print "writing files"
for i in range(0,len(number)):
        timeFile.write(number[i]  + "\t" + time[i]   + "\t" + Wtime[i] + "\n")
        energyFile.write(time[i]  + "\t" + energy[i] + "\n")
        fieldFile.write(time[i]   + "\t" + field[i]  + "\n")
        normFile.write(time[i]    + "\t" + norm[i]   + "\n")
        overlapFile.write(time[i] + "\t" + overlap[i]+ "\n")
        xdipFile.write(time[i]    + "\t" + xdip[i]   + "\n")
        ydipFile.write(time[i]    + "\t" + ydip[i]   + "\n")
        zdipFile.write(time[i]    + "\t" + zdip[i]   + "\n")
        accelFile.write(time[i]   + "\t" + accel[i]  + "\n")
        WtimeFile.write(Wtime[i]  + "\n")
os.system("cp input input.dat")
#os.system("gnuplot -persist ~/bin/makePlots.gp")



import math, sys

# Element info taken mostly from Rasmol but the effective covalent
# radii are from NWChem with those probably coming from Hondo.

cpk = {0: "light grey",
       1: "sky blue",
       2: "red",
       3: "yellow",
       4: "white",
       5: "pink",
       6: "gold",
       7: "blue",
       8: "orange",
       9: "dark grey",
       10: "brown",
       11: "purple",
       12: "deep pink",
       13: "green",
       14: "fire brick",
       15: "mid green"}

# Colors from X11/rgb.txt
colors = {
    "light grey": (211,211,211),
    "sky blue": (135,206,235),
    "red": (255,0,0),
    "yellow": (255,255,0),
    "white": (255,255,255),
    "pink": (255,105,180),
    "gold": (255,215,0),
    "blue": (0,0,255),
    "orange": (255,165,0),
    "dark grey": (169,169,169),
    "brown": (165,42,42),
    "purple": (160,32,240),
    "deep pink": (205,16,118),
    "green": (0,255,0),
    "fire brick": (178,34,34),
    "mid green": (60,179,113)}

for color in colors.keys():
    r,g,b = colors[color]
    colors[color]=r/255.0,g/255.0,b/255.0
    
class Element:
    def __init__(self,symbol,covrad,vdwrad,col,name,atno):
        self.symbol = symbol
        self.covrad = covrad
        self.vdwrad = vdwrad
        self.col    = colors[cpk[col]]
        self.name   = name
        self.atno   = atno

    def __str__(self):
        return "%3d %2s %16s %.1f %.1f (%.2f %.2f %.2f) " % (
            self.atno,self.symbol,self.name,self.covrad,self.vdwrad,
            self.col[0],self.col[1],self.col[2])

elements = range(104)
elements[  0] = Element('Bq' ,0.00, 0.0,  0, "Dummy"    ,  0 )
elements[  1] = Element('H' ,0.30, 2.75,  4, "HYDROGEN"    ,  1 )
elements[  2] = Element('He',1.22, 5.50,  5, "HELIUM"      ,  2 )
elements[  3] = Element('Li',1.23, 3.05, 14, "LITHIUM"     ,  3 )
elements[  4] = Element('Be',0.89, 1.57, 12, "BERYLLIUM"   ,  4 )
elements[  5] = Element('B' ,0.88, 3.87, 13, "BORON"       ,  5 )
elements[  6] = Element('C' ,0.77, 3.87,  9, "CARBON"      ,  6 ) # was color 0
elements[  7] = Element('N' ,0.70, 3.50,  1, "NITROGEN"    ,  7 )
elements[  8] = Element('O' ,0.66, 3.37,  2, "OXYGEN"      ,  8 )
elements[  9] = Element('F' ,0.58, 3.25,  6, "FLUORINE"    ,  9 )
elements[ 10] = Element('Ne',1.60, 5.05, 12, "NEON"        , 10 )
elements[ 11] = Element('Na',1.66, 5.50,  7, "SODIUM"      , 11 )
elements[ 12] = Element('Mg',1.36, 3.75, 15, "MAGNESIUM"   , 12 )
elements[ 13] = Element('Al',1.25, 3.75,  9, "ALUMINIUM"   , 13 )
elements[ 14] = Element('Si',1.17, 5.50,  6, "SILICON"     , 14 )
elements[ 15] = Element('P' ,1.10, 4.70,  8, "PHOSPHORUS"  , 15 )
elements[ 16] = Element('S' ,1.04, 4.52,  3, "SULPHUR"     , 16 )
elements[ 17] = Element('Cl',0.99, 4.37, 13, "CHLORINE"    , 17 )
elements[ 18] = Element('Ar',1.91, 6.92, 12, "ARGON"       , 18 )
elements[ 19] = Element('K' ,2.03, 5.97, 12, "POTASSIUM"   , 19 )
elements[ 20] = Element('Ca',1.74, 4.87,  9, "CALCIUM"     , 20 )
elements[ 21] = Element('Sc',1.44, 3.30, 12, "SCANDIUM"    , 21 )
elements[ 22] = Element('Ti',1.32, 4.87,  9, "TITANIUM"    , 22 )
elements[ 23] = Element('V' ,1.22, 2.65, 12, "VANADIUM"    , 23 )
elements[ 24] = Element('Cr',1.19, 2.82,  9, "CHROMIUM"    , 24 )
elements[ 25] = Element('Mn',1.17, 2.97,  9, "MANGANESE"   , 25 )
elements[ 26] = Element('Fe',1.165,4.87,  8, "IRON"        , 26 )
elements[ 27] = Element('Co',1.16, 2.82, 12, "COBALT"      , 27 )
elements[ 28] = Element('Ni',1.15, 3.10, 10, "NICKEL"      , 28 )
elements[ 29] = Element('Cu',1.17, 2.87, 10, "COPPER"      , 29 )
elements[ 30] = Element('Zn',1.25, 2.87, 10, "ZINC"        , 30 )
elements[ 31] = Element('Ga',1.25, 3.87, 12, "GALLIUM"     , 31 )
elements[ 32] = Element('Ge',1.22, 9.99, 12, "GERMANIUM"   , 32 )
elements[ 33] = Element('As',1.21, 2.07, 12, "ARSENIC"     , 33 )
elements[ 34] = Element('Se',1.17, 2.25, 12, "SELENIUM"    , 34 )
elements[ 35] = Element('Br',1.14, 4.37, 10, "BROMINE"     , 35 )
elements[ 36] = Element('Kr',1.98, 4.75, 12, "KRYPTON"     , 36 )
elements[ 37] = Element('Rb',2.22, 6.62, 12, "RUBIDIUM"    , 37 )
elements[ 38] = Element('Sr',1.92, 5.05, 12, "STRONTIUM"   , 38 )
elements[ 39] = Element('Y' ,1.62, 4.02, 12, "YTTRIUM"     , 39 )
elements[ 40] = Element('Zr',1.45, 3.55, 12, "ZIRCONIUM"   , 40 )
elements[ 41] = Element('Nb',1.34, 3.32, 12, "NIOBIUM"     , 41 )
elements[ 42] = Element('Mo',1.29, 4.37, 12, "MOLYBDENUM"  , 42 )
elements[ 43] = Element('Tc',1.27, 4.50, 12, "TECHNETIUM"  , 43 )
elements[ 44] = Element('Ru',1.24, 3.00, 12, "RUTHENIUM"   , 44 )
elements[ 45] = Element('Rh',1.25, 3.05, 12, "RHODIUM"     , 45 )
elements[ 46] = Element('Pd',1.28, 3.60, 12, "PALLADIUM"   , 46 )
elements[ 47] = Element('Ag',1.34, 3.87,  9, "SILVER"      , 47 )
elements[ 48] = Element('Cd',1.41, 4.37, 12, "CADMIUM"     , 48 )
elements[ 49] = Element('In',1.50, 3.62, 12, "INDIUM"      , 49 )
elements[ 50] = Element('Sn',1.40, 4.17, 12, "TIN"         , 50 )
elements[ 51] = Element('Sb',1.41, 2.80, 12, "ANTIMONY"    , 51 )
elements[ 52] = Element('Te',1.37, 3.15, 12, "TELLURIUM"   , 52 )
elements[ 53] = Element('I' ,1.33, 4.37, 11, "IODINE"      , 53 )
elements[ 54] = Element('Xe',2.09, 5.25, 12, "XENON"       , 54 )
elements[ 55] = Element('Cs',2.35, 7.52, 12, "CAESIUM"     , 55 )
elements[ 56] = Element('Ba',1.98, 6.02,  8, "BARIUM"      , 56 )
elements[ 57] = Element('La',1.69, 4.57, 12, "LANTHANUM"   , 57 )
elements[ 58] = Element('Ce',1.65, 4.65, 12, "CERIUM"      , 58 )
elements[ 59] = Element('Pr',1.65, 4.05, 12, "PRASEODYMIUM", 59 )
elements[ 60] = Element('Nd',1.64, 4.47, 12, "NEODYMIUM"   , 60 )
elements[ 61] = Element('Pm',1.65, 4.40, 12, "PROMETHIUM"  , 61 )
elements[ 62] = Element('Sm',1.66, 4.35, 12, "SAMARIUM"    , 62 )
elements[ 63] = Element('Eu',1.65, 4.90, 12, "EUROPIUM"    , 63 )
elements[ 64] = Element('Gd',1.61, 4.22, 12, "GADOLINIUM"  , 64 )
elements[ 65] = Element('Tb',1.59, 4.15, 12, "TERBIUM"     , 65 )
elements[ 66] = Element('Dy',1.59, 4.07, 12, "DYSPROSIUM"  , 66 )
elements[ 67] = Element('Ho',1.58, 4.02, 12, "HOLMIUM"     , 67 )
elements[ 68] = Element('Er',1.57, 3.97, 12, "ERBIUM"      , 68 )
elements[ 69] = Element('Tm',1.56, 3.92, 12, "THULIUM"     , 69 )
elements[ 70] = Element('Yb',1.56, 3.85, 12, "YTTERBIUM"   , 70 )
elements[ 71] = Element('Lu',1.56, 3.82, 12, "LUTETIUM"    , 71 )
elements[ 72] = Element('Hf',1.44, 3.50, 12, "HAFNIUM"     , 72 )
elements[ 73] = Element('Ta',1.34, 3.05, 12, "TANTALUM"    , 73 )
elements[ 74] = Element('W' ,1.30, 3.15, 12, "TUNGSTEN"    , 74 )
elements[ 75] = Element('Re',1.28, 3.25, 12, "RHENIUM"     , 75 )
elements[ 76] = Element('Os',1.26, 3.95, 12, "OSMIUM"      , 76 )
elements[ 77] = Element('Ir',1.26, 3.05, 12, "IRIDIUM"     , 77 )
elements[ 78] = Element('Pt',1.29, 3.87, 12, "PLATINUM"    , 78 )
elements[ 79] = Element('Au',1.34, 3.62,  6, "GOLD"        , 79 )
elements[ 80] = Element('Hg',1.44, 4.95, 12, "MERCURY"     , 80 )
elements[ 81] = Element('Tl',1.55, 4.27, 12, "THALLIUM"    , 81 )
elements[ 82] = Element('Pb',1.54, 5.40, 12, "LEAD"        , 82 )
elements[ 83] = Element('Bi',1.52, 4.32, 12, "BISMUTH"     , 83 )
elements[ 84] = Element('Po',1.53, 3.02, 12, "POLONIUM"    , 84 )
elements[ 85] = Element('At',1.50, 2.80, 12, "ASTATINE"    , 85 )
elements[ 86] = Element('Rn',2.20, 5.75, 12, "RADON"       , 86 )
elements[ 87] = Element('Fr',3.24, 8.10, 12, "FRANCIUM"    , 87 )
elements[ 88] = Element('Ra',2.68, 6.42, 12, "RADIUM"      , 88 )
elements[ 89] = Element('Ac',2.25, 5.30, 12, "ACTINIUM"    , 89 )
elements[ 90] = Element('Th',2.16, 4.60, 12, "THORIUM"     , 90 )
elements[ 91] = Element('Pa',1.93, 4.00, 12, "PROTACTINIUM", 91 )
elements[ 92] = Element('U' ,3.00, 4.37, 12, "URANIUM"     , 92 )
elements[ 93] = Element('Np',1.57, 4.27, 12, "NEPTUNIUM"   , 93 )
elements[ 94] = Element('Pu',1.81, 4.17, 12, "PLUTONIUM"   , 94 )
elements[ 95] = Element('Am',2.21, 4.15, 12, "AMERICIUM"   , 95 )
elements[ 96] = Element('Cm',1.43, 4.12, 12, "CURIUM"      , 96 )
elements[ 97] = Element('Bk',1.42, 4.10, 12, "BERKELIUM"   , 97 )
elements[ 98] = Element('Cf',1.40, 4.07, 12, "CALIFORNIUM" , 98 )
elements[ 99] = Element('Es',1.39, 4.05, 12, "EINSTEINIUM" , 99 )
elements[100] = Element('Fm',1.38, 4.02, 12, "FERMIUM"     ,100 )
elements[101] = Element('Md',1.37, 4.00, 12, "MENDELEVIUM" ,101 )
elements[102] = Element('No',1.36, 3.97, 12, "NOBELIUM"    ,102 )
elements[103] = Element('Lr',1.34, 3.95, 12, "LAWRENCIUM"  ,103 )

default_radii = range(104)
default_colors = range(104)
for i in range(1,len(elements)):
    default_radii[i]  = elements[i].covrad
    default_colors[i] = elements[i].col
    
def scale_coords(coords, fac):
    natom = len(coords)
    for atom in range(natom):
        for i in range(3):
            coords[atom][i] = coords[atom][i]*fac

def dist(i,j):
    return math.sqrt((i[0]-j[0])**2+
                     (i[1]-j[1])**2+
                     (i[2]-j[2])**2)

def make_bonds(coords, z, radii):
    bonds = []
    for i in range(len(coords)):
        for j in range(i):
            rij = dist(coords[i],coords[j])
            if rij < 1.1*(radii[z[i]]+radii[z[j]]):
                #print "bond",i,j,rij
                bonds.append([i,j])
    return bonds

def moldx(coords,z,bonds=None,units="angstrom",fname="mol.dx",
          radii=default_radii,colors=default_colors,
          scale=None,shift=None):
    '''

    Make an Opendx format file with a stick representation of the
    molecule.  Additional representations will be easy to add.

    coords = coords[natom][3] input coords (unchanged on output)

    z = z[natom] input atomic number of each atom (unchanged on output)

    bonds = optional list of precomputed connections.  If None, these
    .       are computed from the covalent radii.

    units = the input units.  If the input units are atomic, they will be
    .       converted to angstrom.  The output units are always angstrom
    .       unless additional scaling is specified.

    fname = the name of the output file

    radii = covalent radii for forming bonds.  Defaults will be used
    .       if not speficied.

    colors = colors for the atoms (3-vector float in [0,1]).  Defaults
    .        will be used if not specified.

    scale = if scale (scalar float) is specified, the *output*
    .       coordinates are scaled by that factor.

    shift = if shift (3-vector float) is specified, the *output*
    .       coordinates are shifted *after* scaling
    .       (e.g., x*scale+shift[0])

    '''

    au_to_angs = 0.529177249
    if len(z) != len(coords): raise ValueError

    if not bonds:
        # Need angstroms for making bonds
        if units in ["au","a.u.","atomic","bohr"]:
            scale_coords(coords,au_to_angs)

        bonds = make_bonds(coords,z,radii)

        if units in ["au","a.u.","atomic","bohr"]:
            scale_coords(coords,1.0/au_to_angs)

    # We have the atomic coordinates, numbers, radii, and
    # connectivity.  The opendx stick representation is formed
    # by splitting the bonds at the midpoint (weighted by
    # the ratio of the radii) with appropriate coloring.

    if not units in ["au","a.u.","atomic","bohr"]:
        # Need atomic units for the output
        scale_coords(coords,1.0/au_to_angs)

    pos = []
    col = []
    con = []
    
    for i,j in bonds:
        npos = len(pos)
        rij = dist(coords[i],coords[j])
        si = radii[z[i]]/(radii[z[i]]+radii[z[j]])
        sj = 1.0-si
        mid = [coords[i][0]*sj+coords[j][0]*si,
               coords[i][1]*sj+coords[j][1]*si,
               coords[i][2]*sj+coords[j][2]*si]
        
        pos.append(coords[i]); col.append(colors[z[i]])
        pos.append(mid); col.append(colors[z[i]])

        pos.append(mid); col.append(colors[z[j]])
        pos.append(coords[j]); col.append(colors[z[j]])

        con.append([npos,npos+1])
        con.append([npos+2,npos+3])

    f = open(fname,'w+')

    f.write("object 1 class array type float rank 1 shape 3 items %d data follows\n" % len(pos))
    for x,y,z in pos:
        if scale: x, y, z = x*scale, y*scale, z*scale
        if shift: x, y, z = x+shift[0], y+shift[1], z+shift[2]
        f.write("%f %f %f\n" % (x,y,z))
    f.write('attribute "dep" string "positions"\n')

    f.write("object 2 class array type int rank 1 shape 2 items %d data follows\n" % len(con))
    for i,j in con: f.write("%d %d \n" % (i,j))
    f.write('attribute "element type" string "lines"\n')
    f.write('attribute "ref" string "positions"\n')
    f.write('attribute "dep" string "connections"\n')

    f.write("object 3 class array type float rank 0 items %d data follows\n" % len(pos))
    for p in pos: f.write("0 ")
    f.write("\n")
    f.write('attribute "dep" string "positions"\n')

    f.write("object 4 class array type float rank 1 shape 3 items %d data follows\n" % len(col))
    for r,g,b in col: f.write("%f %f %f\n" % (r,g,b))
    f.write('attribute "dep" string "positions"\n')

    f.write('object "irregular positions irregular connections" class field\n')
    f.write('component "positions" value 1\n')
    f.write('component "connections" value 2\n')
    f.write('component "data" value 3\n')
    f.write('component "colors" value 4\n')
    f.write('attribute "name" string "irregular positions irregular connections"\n')
    f.write('end\n')

    f.close()

    # Undo-Force a.u. while making the bonds (cannot do until after
    # writing file since pos contains references to these values).
    if not units in ["au","a.u.","atomic","bohr"]:
        scale_coords(coords,au_to_angs)
    

def symtoatn(sym):
    sym = sym.lower()
    for el in elements:
        if el.symbol.lower() == sym:
            return el.atno
    raise "Uh?"

if __name__ == "__main__":
    # Read first molecule from the madness output file
    f = sys.stdin
    while 1:
        line = f.readline().split()
        if len(line) and line[0] == 'geometry':
            coords = []
            z = []
            units = 'au'
            while 1:
                line = f.readline().split()
                if len(line) == 0: continue
                if line[0] == 'end':
                    break
                elif line[0] == 'units':
                    units = line[1]
                elif not (line[0] == 'units' or line[0] == 'eprec'):
                    z.append(symtoatn(line[0]))
                    coords.append([float(line[1]), float(line[2]), float(line[3])])
            moldx(coords, z, units=units, fname='molecule.dx')
            sys.exit(0)

    



#!/usr/bin/env python3

import fileinput

class COLORS:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'

    def disable(self):
        self.HEADER = ''
        self.OKBLUE = ''
        self.OKGREEN = ''
        self.WARNING = ''
        self.FAIL = ''
        self.ENDC = ''

# { (shape, i, j, k) : [(variant, gflops), ] , }

def get_data():
    results = {}
    for line in fileinput.input():
        line = line.split()
        test = tuple([line[1]] + [int(x) for x in line[2:-1]])
        gflops = float(line[-1])
        version = line[0][:-1] # Remove semicolon
        if test not in results:
            results[test] = []
        results[test].append((version, gflops))
    return results

def print_best_per_benchmark(results):
    best_avg, blas_avg, imp_avg = 0, 0, 0
    for test in sorted(results):
        best = max(results[test], key=lambda x: x[1])
        best_avg += best[1]
        color = COLORS.OKGREEN
        print(color + '{:40}'.format(' '.join([str(x) for x in test])) + COLORS.ENDC, '{:40}{:6.2f}'.format(*best))
    best_avg /= len(results)
    print('{:40} {:40}{:6.2f}'.format('', 'average', best_avg))

def print_top_n_variants(results, n=10):
    variants = {}
    for test in results:
        for variant, gflops in results[test]:
            if variant not in variants:
                variants[variant] = 0
            variants[variant] += gflops / len(results)

    topn = sorted(variants, key=lambda x: variants[x], reverse=True)[:n]
    for x in topn:
        print('{:40}{:6.2f}'.format(x, variants[x]))


if __name__ == '__main__':
    results = get_data()
    print(COLORS.HEADER + "Best per benchmark:" + COLORS.ENDC)
    print_best_per_benchmark(results)
    print(COLORS.HEADER + "Top 10 Varaints:" + COLORS.ENDC)
    print_top_n_variants(results)



import unittest
import doctest
import sys
import os

def grep(filename, string):
    """
    Input:
        A filename to read and a string to search for.

    Output:
        True if the string appears in the file
        False otherwise
    """
    for line in open(filename):
        if string in line:
            return True
    return False

def load_tests(loader, tests, ignore):
    """
    Import files with doctests into the unittesting framework.
    """
    # Find the base of the project 
    base = sys.path[0]

    # Walk all files in this project
    for dirpath, dirnames, filenames in os.walk(base):
        for fn in filenames:
            # Check python files for the doctest prompt
            if fn[-3:] == ".py" and grep(os.path.join(dirpath,fn), '>'*3):
                # remove the project name and .py
                module = os.path.join(dirpath[len(base):], fn[:-3])
                # use '.' to make it a string that represents a module
                module = module.replace(os.sep,'.').strip('.')
                # add the module as a test
                tests.addTests(doctest.DocTestSuite(module))

    return tests



def _tester(i, m, trans=False):
    ret = """
    if (myid=={i}) {{
#ifdef HAVE_BGP
        HPM_Start("{m}");
#endif
        for (loop=0; loop<30; ++loop) {{
            double rate;
            start_timer();
            {m}(ni,nj,nk,c,a,b);"""
    if trans:
        ret += """
            {m}(ni,nj,nk,a,c,b);
            {m}(ni,nj,nk,c,a,b);"""
    ret +="""
            unsigned long long tt = stop_timer();
            rate = nflop/tt;
            if (rate > fastest) fastest = rate;
        }}

#ifdef HAVE_BGP
        HPM_Stop("{m}");
        MPI_Barrier(MPI_COMM_WORLD);
#endif
        printf("{m}: %20s %3ld %3ld %3ld %8.2f\\n", s, ni, nj, nk, fastest);
#ifdef HAVE_BGP
        MPI_Barrier(MPI_COMM_WORLD);
#endif
    }}
"""
    return ret.format(i=i, m=m)


def _timer(mtxms, complex_c, complex_a, complex_b):
    ret = """void timer(const char* s, long ni, long nj, long nk, double {} *a, double {} *b, double {} *c) {{
    double fastest = 0.0;
    double nflop = 2.0{}{}*ni*nj*nk;
    long loop;
""".format(complex_a, complex_b, complex_c, complex_a and "*2.0", complex_b and "*2.0")

    for i, m in enumerate(mtxms):
        ret += _tester(i, m)

    ret += "}\n"
    return ret


def _transtimer(mtxms, complex_c, complex_a, complex_b):
    ret = """void trantimer(const char* s, long ni, long nj, long nk, double {} *a, double {} *b, double {} *c) {{
    double fastest = 0.0;
    double nflop = 3.0*2.0{}{}*ni*nj*nk;
    long loop;
""".format(complex_a, complex_b, complex_c, complex_a and "*2.0", complex_b and "*2.0")

    for i, m in enumerate(mtxms):
        ret += _tester(i, m, True)

    ret += "}\n"
    return ret


def _header(bg, complex_c, complex_a, complex_b):
    ret = ""
    if bg:
        ret += "#define HAVE_BGP 1\n"
    ret += """#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#ifdef HAVE_BGP
#include <builtins.h>
#include <complex>

#include "mpi.h"

extern "C" void HPM_Init(void);           // initialize the UPC unit
extern "C" void HPM_Start(char *label);   // start counting in a block marked by the label
extern "C" void HPM_Stop(char *label);    // stop counting in a block marked by the label
extern "C" void HPM_Print(void);          // print counters for all blocks
extern "C" void HPM_Print_Flops(void);

#else

#include <time.h>
#include <immintrin.h>
#include <complex.h>

#endif

int numprocs, myid;

double ran()
{
    static unsigned long seed = 76521;
    seed = seed *1812433253 + 12345;
    return ((double) (seed & 0x7fffffff)) * 4.6566128752458e-10;
}

void ran_fill(int n, double *a) {
    while (n--) *a++ = ran();
}

void mtxm(long dimi, long dimj, long dimk,
    """ + "double {} * c, const double {} * a, const double {} * b)".format(complex_c, complex_a, complex_b) + """ {
    int i, j, k;
    for (k=0; k<dimk; ++k) {
        for (j=0; j<dimj; ++j) {
            for (i=0; i<dimi; ++i) {
                c[i*dimj+j] += a[k*dimi+i]*b[k*dimj+j];
            }
        }
    }
}

unsigned long long _ts;

#if HAVE_BGP 

static __inline__ unsigned long long rdtsc(void)
{
  unsigned long long int result=0;
  unsigned long int upper, lower,tmp;
  __asm__ volatile(
                "0:            "
                "mftbu   %0    "
                "mftb    %1    "
                "mftbu   %2    "
                "cmpw    %2,%0 "
                "bne     0b    "
                : "=r"(upper),"=r"(lower),"=r"(tmp)
                );
  result = upper;
  result = result<<32;
  result = result|lower;

  return(result);
}

#else

static __inline__ unsigned long long rdtsc(void)
{
    unsigned long long int x;
    __asm__ volatile ("rdtsc; shlq $32, %%rdx; orq %%rdx, %0" : "=a" (x) : : "rdx");
    return x;
}

#endif

inline void start_timer() {
    _ts = rdtsc();
}

inline unsigned long long stop_timer() {
    return rdtsc() - _ts;
}
"""
    return ret

def _main(mtxms, complex_c, complex_a, complex_b):
    ret = """int main(int argc, char **argv) {
    const long nimax=30*30;
    const long njmax=100;
    const long nkmax=100;
    long ni, nj, nk, i, m;

    """ + "double {} *a;".format(complex_a) + """
    """ + "double {} *b;".format(complex_b) + """
    """ + "double {} *c;".format(complex_c) + """
    """ + "double {} *d;".format(complex_c) + """

#ifdef HAVE_BGP
    MPI_Init(&argc, &argv);

    MPI_Comm_size(MPI_COMM_WORLD,&numprocs);
    MPI_Comm_rank(MPI_COMM_WORLD,&myid);

    HPM_Init();
#else
    numprocs = 1;
    myid = 0;
#endif

    posix_memalign((void **) &a, 16, nkmax*nimax*sizeof(double """ + complex_a + """));
    posix_memalign((void **) &b, 16, nkmax*njmax*sizeof(double """ + complex_b + """));
    posix_memalign((void **) &c, 16, nimax*njmax*sizeof(double """ + complex_c + """));
    posix_memalign((void **) &d, 16, nimax*njmax*sizeof(double """ + complex_c + """));

    ran_fill(nkmax*nimax""" + (complex_a and "*2") + """, (double*)a);
    ran_fill(nkmax*njmax""" + (complex_a and "*2") + """, (double*)b);

    for (ni=2; ni<60; ni+=2) {
        for (nj=2; nj<100; nj+=6) {
            for (nk=2; nk<100; nk+=6) {
                for (i=0; i<ni*nj; ++i) d[i] = c[i] = 0.0;
                mtxm(ni,nj,nk,c,a,b);
"""
    for i, f in enumerate(mtxms):
        ret += "                if (myid == {}) {}(ni,nj,nk,d,a,b);\n".format(i, f)
    ret += """
                for (i=0; i<ni*nj; ++i) {
                    double err = """ + (complex_c and "c" or "f") + """abs(d[i]-c[i]);
                    if (err > 1e-""" + (complex_c and "12" or "14") +""") {
"""
    for i, f in enumerate(mtxms):
        ret += """                        if (myid == {}) fprintf(stderr, "{} error %d: %ld %ld %ld %e\\n",myid,ni,nj,nk,err);\n""".format(i, f)
    ret += """
                    }
                }
            }
        }
    }
#ifdef HAVE_BGP
    MPI_Barrier(MPI_COMM_WORLD);
#endif

    for (ni=2; ni<60; ni+=2) timer("(m*m)T*(m*m)", ni,ni,ni,a,b,c);
    for ( m=2; m<=30;  m+=2) timer("(m*m,m)T*(m*m)", m*m,m,m,a,b,c);
    """ + ((not (bool(complex_a) ^ bool(complex_b))) and """for ( m=2; m<=30;  m+=2) trantimer("tran(m,m,m)", m*m,m,m,a,b,c);""" or "") + """
    for ( m=2; m<=20;  m+=2) timer("(20*20,20)T*(20,m)", 20*20,m,20,a,b,c);

#ifdef HAVE_BGP
    HPM_Print();
    HPM_Print_Flops();

    MPI_Finalize();
#endif

    return 0;
}"""
    return ret

def tester_gen(f, mtxmspecs, mtxm_gen, complex_a=False, complex_b=False, bg=True):
    r = lambda x: x
    if bg:
        r = lambda x: x.replace("double complex", "__complex__ double").replace("cabs", "abs")
    complex_c = (complex_a or complex_b) and "complex" or ""
    complex_a = complex_a and "complex" or ""
    complex_b = complex_b and "complex" or ""
    print(r(_header(bg, complex_c, complex_a, complex_b)), file=f)
    for mtxm in mtxmspecs:
        mtxm_gen(f, *mtxm)
    mtxms = [x[-1] for x in mtxmspecs]
    print(r(_timer(mtxms, complex_c, complex_a, complex_b)), file=f)
    if not (bool(complex_a) ^ bool(complex_b)):
        print(r(_transtimer(mtxms, complex_c, complex_a, complex_b)), file=f)
    print(r(_main(mtxms, complex_c, complex_a, complex_b)), file=f)



#!/usr/bin/python
import sys, re, os, os.path
try:
    sys.argv[1]
except:
    print "Which file? "
    sys.exit("tellme")
inputFile = sys.argv[1]
if not os.path.isfile(inputFile):
    print inputFile, "doesn't exist"
    sys.exit()
print inputFile

#Parse projPsi's output 
writeEX  = False
writeION = False
writeY   = False
f = open(inputFile, 'r')
lines = f.readlines()
#only opening files for which we have data
for line in lines:
    exState = re.match( "^[1-9]", line)
    kState  = re.match( "^0", line)
    Yl      = re.match( "^Y(\d)0", line)
    # If Y is matched, don't match EX or ION
    if Yl:
        writeY = True
        break
    # Can match EX and/or ION
    if exState:
        writeEX = True
    if kState:
        writeION = True
if writeEX:
    exFile = open("ex.dat", 'w')
    print "matchEX"
if writeION:
    ionFile  = open("ion.dat", 'w')
    unboundFile = open("unbound.num", 'r')
    kList    = unboundFile.readlines()
    unboundFile.close()
    print "matchION"
if writeY:
    RlFile = open("Rl.dat", 'w')
    print "matchY"
while 1:
    if(lines):
        line = lines.pop(0)
        exState = re.match( "^[1-9]", line)
        kState = re.match( "^0", line)
        Yl = re.match( "^Y(\d)0", line)
        if exState:
            words = line.split()
            exFile.write(' '.join(words) + '\n')
        if kState:
            words = line.split()
            words.pop(0)
            kList.pop(0) #Trim unbound.num
            for word in words:
                if( word == 'took'):
                    break
                ionFile.write(word + "\t")
            ionFile.write("\n")
        if Yl:
            words  = line.split()
            l  =  Yl.group(1)
            Pl =  lines.pop(0)[0:-1]
            RlFile.write( l + "\t" + Pl + "\t" + " ".join(words[1:]) + "\n" )
    else:
        break
f.close()
if writeEX:
    exFile.close()
if writeION:
    ionFile.close()
if writeY:
    RlFile.close()
unboundFile = open("unbound.num", 'w')
for line in kList:
    unboundFile.write(line)
unboundFile.close()

#Sort STEP, atomic time, and walltime
f = open("time.dat", 'r')
lines = f.readlines()
legacyTime = 0
lastTime  = 0
lastStep  = 0
fout = open("t.dat", "w")
for line in lines:
    time = line.split()
    step  = float(time[0]) # time step number
    qTime = time[1]        # quantum time
    wTime = float(time[2]) # wall time
    if step <= lastStep:   # Don't allow repeat time steps
        continue
    if wTime < lastTime:   # add to legacy time
        legacyTime += lastTime
    totalTime = wTime + legacyTime
    fout.write(time[0] + "\t" + time[1] + "\t" + str(totalTime) + "\n")
    lastStep = step
    lastTime = wTime
f.close()
tFile = open("tMAX.dat", 'w')
tFile.write( qTime )
tFile.close()

if os.path.isfile('input'):
    os.system('cp input input.dat')
if os.path.isfile('input2'):
    os.system("cp input2 input2.dat")
#make dr.dat
if( writeY ):
    thisDIR = os.getcwd()
    inputFile = thisDIR  + '/input'
    input2File = thisDIR + '/input2'
    if os.path.isfile(inputFile):
        f = open("input", 'r')
        lines = f.readlines()
        for line in lines:
            if len(line) == 1:
                continue
            word = line.split()
            if word[0] == 'L':
                L = float(word[1])
        f.close()
        if os.path.isfile(input2File):
            f = open("input2", 'r')
            lines = f.readlines()
            for line in lines:
                if line:
                    word = line.split()
                    if word[0] == 'nGrid':
                        n = int(word[1])
            f.close()
            f = open("dr.dat", 'w')
            f.write( str(L/n) )
            f.close()



""" Codegen for mtxm

A   I
  +-------------+
K | a b ....... |
  |     ...     |
  +-------------+

B   J
  +-------------+
K | i j k l ... |
  |     ...     |
  +-------------+

C   J
  +-------------+
I | w x y z ... |
  |     ...     |
  +-------------+

# Complex Complex
w += a i - b j
x += a j + b i
y += a k - b l
z += a l + b k

temps: _c += _a _b +- _ai _br


# Real Complex
w += a i
x += a j
y += a k
z += a l

temps: _c += _a _b ; doubled dimj


# Complex Real
w += a i
x += b i
y += a j
z += b j

temps: _c += _az _bz


# Real Real
w += a i
x += a j
y += a k
z += a l

temps: _c += _a _b
"""

from itertools import product
import logging
logger = logging.getLogger(__name__)

class MTXMGen:
    def __init__(self, cxa=False, cxb=False):
        self.indent = 4
        self.__in_main_loop = False
        self._mask = False
        self._odds = [1]
        self.have_bgp = False
        self.have_bgq = False
        self.have_avx2 = False
        self.complex_a = cxa
        self.complex_b = cxb
        self.complex_dup_cast = ''

    @property
    def complex_c(self):
        return self.complex_a or self.complex_b

    @property
    def complex_complex(self):
        return self.complex_a and self.complex_b

    @property
    def real_complex(self):
        return not self.complex_a and self.complex_b

    @property
    def complex_real(self):
        return self.complex_a and not self.complex_b

    @property
    def real_real(self):
        return not self.complex_a and not self.complex_b

    def _temp(self, prefix, x, y):
        return prefix + '_' + str(x) + '_' + str(y)

    def _temps(self, prefix, x, y, size):
        """
        >>> list(MTXMGen()._temps('_x', 'i', 'j', {'i':2, 'j':3}))
        ['_x_0_0', '_x_0_1', '_x_0_2', '_x_1_0', '_x_1_1', '_x_1_2']
        """
        return [self._temp(prefix, i, j) for i, j in product(range(size[x]), range(size[y]))]

    def _post_process(self, lines):
        return lines

    def _header(self, func_name):
        f = lambda x: x and "complex" or ""
        x2 = lambda x: x and "*2" or ""
        if self.have_bgq:
            ret = ["void " + func_name + "(long dimi, long dimj, long dimk, long extb, double {} * __restrict__ c_x, const double {} * __restrict__ a_x, const double {} * __restrict__ b_x) {{".format(f(self.complex_c), f(self.complex_a), f(self.complex_b))]
        else:
            ret = ["void " + func_name + "(long dimi, long dimj, long dimk, double {} * __restrict__ c_x, const double {} * __restrict__ a_x, const double {} * __restrict__ b_x) {{".format(f(self.complex_c), f(self.complex_a), f(self.complex_b))]
        ret.append("    int i, j, k;")
        ret.append("    double * __restrict__ c = (double*)c_x;")
        ret.append("    const double * __restrict__ a = (double*)a_x;")
        ret.append("    const double * __restrict__ b = (double*)b_x;")
        ret.append("    long effj = dimj;")
        if self.have_bgq:
            ret.append("    double * c_buf;")
            ret.append("    double * b_buf;")
            ret.append("    bool free_b = false;")
            ret.append("    /* Setup a buffer for c if needed */")
            ret.append("    double* c_out = c;")
            ret.append("    if (dimj%{}) {{".format(self.complex_c and 2 or 4))
            ret.append("        effj = (dimj | {}) + 1;".format(self.complex_c and 1 or 3))
            ret.append("        posix_memalign((void **) &c, 32, dimi*effj*sizeof(double){});".format(x2(self.complex_c)))
            ret.append("        c_buf = c;")
            ret.append("    }")
            ret.append("    /* Copy b into a buffer if needed */")
            ret.append("    if (extb%{}) {{".format(self.complex_b and 2 or 4))
            ret.append("        long t_extb = (dimj | {}) + 1;".format(self.complex_b and 1 or 3))
            ret.append("        free_b = true;")
            ret.append("        posix_memalign((void **) &b_buf, 32, dimk*t_extb*sizeof(double){});".format(x2(self.complex_b)))
            ret.append("        double* bp = b_buf;")
            ret.append("        for (k=0; k<dimk; k++, bp += t_extb{0}, b += extb{0})".format(x2(self.complex_b)))
            ret.append("            memcpy(bp, b, sizeof(double)*dimj{});".format(x2(self.complex_b)))
            ret.append("        b = b_buf;")
            ret.append("        extb = t_extb;")
            ret.append("    }")
        return ret

    def _footer(self):
        x2 = lambda x: x and "*2" or ""
        ret = []
        if self.have_bgq:
            ret.append("    /* Copy c out if needed */")
            ret.append("    if (dimj%{}) {{".format(self.complex_c and 2 or 4))
            ret.append("        double* ct = c_buf;")
            ret.append("        for (i=0; i<dimi; i++, ct += effj{0}, c_out += dimj{0})".format(x2(self.complex_c)))
            ret.append("            memcpy(c_out, ct, sizeof(double)*dimj{});".format(x2(self.complex_c)))
            ret.append("        free(c_buf);")
            ret.append("    }")
            ret.append("    /* Free the buffer for b */")
            ret.append("    if (free_b) free(b_buf);")
        ret.append("}")
        return ret

    def _temp_dec(self, size):
        ret = []
        indent = '    '
        x = indent + self.vector_type + ' '
        x += ', '.join(self._temps('_c', 'i', 'j', size) +
                self._temps('_b', 'k', 'j', size)) + ';'
        ret.append(x)
        x = indent + self.splat_type + ' ' + ', '.join(self._temps('_a', 'k', 'i', size)) + ';'
        ret.append(x)
        if self.complex_complex:
            if not self.have_bgp and not self.have_bgq:
                # BGP does not need seperate reversed registers because a special fma is used
                x = "{} {} {};".format(indent, self.vector_type, ', '.join(self._temps('_br', 'k', 'j', size)))
                ret.append(x)
            if not self.have_bgq:
                # Imaginary component of A
                x = "{} {} {};".format(indent, self.splat_type, ', '.join(self._temps('_ai', 'k', 'i', size)))
                ret.append(x)
        elif self.complex_real:
            # register from A: a b a b
            x = "{} {} {};".format(indent, self.vector_type, ', '.join(self._temps('_az', 'k', 'i', size)))
            ret.append(x)
            # register from B: i i j j
            x = "{} {} {};".format(indent, self.splat_type, ', '.join(self._temps('_bz', 'k', 'j', size)))
            ret.append(x)
        elif self.real_complex:
            pass
        return ret

    def _extra(self):
        return []

    def _temps_to_load(self, unrolls, z, x, y, tname=None):
        if not tname:
            tname = '_'+z
        ret = []
        ystep = 1
        if y == 'j':
            ystep = self.vector_length
        for i, j in product(range(unrolls[x]), range(0, unrolls[y], ystep)):
            ret.append((self._temp(tname, i, j), i, j))
        return ret

    def _load_a(self, unrolls, indent):
        spaces = ' ' * (self.indent*indent)
        ret = []
        for temp, k, i in self._temps_to_load(unrolls, 'a', 'k', 'i'):
            addr = '(pa+' + str((self.complex_a and 2 or 1)*i) + ')'
            if self.complex_real:
                ret.append(self._load_az(spaces, addr, temp, k, i))
            elif self.complex_complex and self.have_bgq:
                ret.append(spaces + temp + " = vec_ld2(0, {});".format(addr))
            else:
                arg0 = ''
                if self.have_bgq:
                    arg0 = '0, '
                ret.append(spaces + temp + ' = {}({}{});'.format(self.splat_op, arg0, addr))
                if self.complex_complex:
                    ret.append(spaces + self._temp('_ai', k, i) + ' = {}({}{}+1);'.format(self.splat_op, arg0, addr))
        return ret

    def _load_b(self, unrolls, indent):
        spaces = ' ' * (self.indent*indent)
        ret = []
        for temp, k, j in self._temps_to_load(unrolls, 'b', 'k', 'j'):
            arg0 = ""
            if self.have_bgq:
                arg0 = "0, "
            addr = '({}pb+{})'.format(arg0, j // (self.complex_real and 2 or 1))
            if self.complex_real:
                ret.append(self._load_bz(spaces, addr, temp, k, j))
            else:
                ret.append(spaces + temp + ' = ' + self.vector_load + addr + ';')
                if self.complex_complex and not self.have_bgp and not self.have_bgq:
                    ret.append(self._load_br(spaces, addr, temp, k, j))
        return ret

    def _load_c(self, unrolls, indent):
        spaces = ' ' * (self.indent*indent)
        ret = []
        for temp, i, j in self._temps_to_load(unrolls, 'c', 'i', 'j'):
            ret.append(spaces + temp + ' = ' + self.vector_zero + ';')
        return ret

    def _load_br(self, spaces, addr, temp, k, j):
        return spaces + self._temp('_br', k, j) + ' = {}({});'.format(self.complex_reverse_dup, addr)

    def _load_az(self, spaces, addr, temp, k, i):
        arg0 = ''
        if self.have_bgq:
            arg0 = '0, '
        return spaces + self._temp('_az', k, i) + ' = {}({}{}{});'.format(self.complex_dup, arg0, self.complex_dup_cast, addr)

    def _load_bz(self, spaces, addr, temp, k, j):
        return spaces + self._temp('_bz', k, j) + ' = {}({});'.format(self.pair_splat, addr)

    def _fma(self, at, bt, ct):
        raise NotImplementedError()

    def _fmaddsub(self, at, bt, ct):
        raise NotImplementedError()

    def _maths(self, unrolls, indent=0):
        spaces = ' ' * (self.indent*indent)
        ret = []
        for j, i, k in product(range(0, unrolls['j'], self.vector_length), range(unrolls['i']), range(unrolls['k'])):
            if self.real_real or self.real_complex:
                at = self._temp('_a', k, i)
                bt = self._temp('_b', k, j)
                ct = self._temp('_c', i, j)
                ret.append(spaces + self._fma(at, bt, ct))
            elif self.complex_real:
                at = self._temp('_az', k, i)
                bt = self._temp('_bz', k, j)
                ct = self._temp('_c', i, j)
                ret.append(spaces + self._fma(at, bt, ct))
            elif self.complex_complex:
                if self.have_avx2:
                    at = self._temp('_ai', k, i)
                    bt = self._temp('_br', k, j)
                    ct = self._temp('_c', i, j)
                    ret.append(spaces + self._fmaddsub(at, bt, ct))
                    at = self._temp('_a', k, i)
                    bt = self._temp('_b', k, j)
                    ret.append(spaces + self._fmaddsub(at, bt, ct))
                else:
                    at = self._temp('_a', k, i)
                    bt = self._temp('_b', k, j)
                    ct = self._temp('_c', i, j)
                    ret.append(spaces + self._fma(at, bt, ct))
                    if not self.have_bgq:
                        at = self._temp('_ai', k, i)
                        if not self.have_bgp:
                            bt = self._temp('_br', k, j)
                    ret.append(spaces + self._fmaddsub(at, bt, ct))
        return ret

    def _array(self, z, x, xx, y, yy, cpx):
        if y == 'j':
            y = "effj"
        else:
            y = "dim" + y

        return z + '+(' + x + '+' + xx + ')*' + y + (cpx and "*2" or "") + '+' + yy

    def _store_c(self, unrolls, indent, bc_mod=""):
        spaces = ' ' * (self.indent*indent)
        ret = []
        jstep = self.vector_length
        for i, j in product(range(unrolls['i']), range(0, unrolls['j'], jstep)):
            if j + jstep < unrolls['j'] or self.__in_main_loop or not self._mask:
                arg0 = self._array(bc_mod+'c', 'i', str(i), 'j', str(j), self.complex_c)
                arg1 = self._temp('_' + 'c', i, j)
                mid = ', '
                if self.have_bgq:
                    arg0, arg1 = arg1, arg0
                    mid = ', 0, '
                ret.append(spaces + self.vector_store + '(' + arg0 + mid + arg1 + ');')
            else:
                # This is somewhat AVX specific, but no other arch's currently support masking, so ok.
                ret.append(spaces + '{}('.format(self.mask_store) + self._array(bc_mod+'c', 'i', str(i), 'j', str(j), self.complex_c) + ', mask, ' + self._temp('_' + 'c', i, j) + ');')
        return ret

    def _loops(self, i, size, bc_mod=""):
        if i == 'i':
            start = 'i=0'
            #FIXME Don't include _odds if i%2==0 and only evens
            loops = [size[i]]
            if loops[-1] != 1:
                loops += self._odds
            if self.have_bgp:
                loops = range(size[i], 0, -2)
            for loop in loops:
                yield ('for ({0}; i+{1}<=dimi; i+={1}) {{'.format(start, loop), loop)
                start = ''
        elif i == 'j':
            loop = size[i] // (self.complex_c and 2 or 1)
            self.__in_main_loop = True
            yield ("for (j=effj; j>{0}; j-={0},{1}c+={0}{2},{1}b+={0}{3}) {{".format(loop, bc_mod, self.complex_c and "*2" or "", self.complex_b and "*2" or ""), size[i])
            self.__in_main_loop = False
            start = ''
            for loop in range(size[i]-self.vector_length, 0, -self.vector_length):
                yield (start + "if (j>{}) {{".format(loop//(self.complex_c and 2 or 1)), loop+self.vector_length)
                start = 'else '
            if size[i] == self.vector_length:
                yield ("{", self.vector_length)
            else:
                yield ("else {", self.vector_length)
        elif i == 'k':
            assert(size[i] == 1)
            pb_inc  = 'effj'
            if self.have_bgq:
                pb_inc = 'extb'
            yield ("for (k=0; k<dimk; k+=1,pb+={}{},pa+=dimi{}) {{".format(pb_inc, self.complex_b and "*2" or "", self.complex_a and "*2" or ""), 1)

    def _close_braces(self, indent=0):
        ret = []
        for i in range(indent, -1, -1):
            ret += [' '*(self.indent*i) + '}']
        return ret

    def _inner_loops(self, perm, sizes, indent=0, unrolls=None, bc_mod=""):
        indent += 1
        if not unrolls:
            unrolls = {x:0 for x in perm}
        ret = []
        spaces = ' '*(self.indent*indent)

        if perm == ['k']:
            ret.append(spaces + "const double* __restrict__ pb = {}b;".format(bc_mod))
            ret.append(spaces + "const double* __restrict__ pa = a+i{};".format(self.complex_a and "*2" or ""))
            ret += self._load_c(unrolls, indent)

        if perm == ['j', 'k']:
            bc_mod = "x"
            ret.append(spaces + "const double* __restrict__ {}b = b;".format(bc_mod))
            ret.append(spaces + "double* __restrict__ {}c = c;".format(bc_mod))

        for loop, unroll in self._loops(perm[0], sizes, bc_mod):
            unrolls[perm[0]] = unroll
            ret.append(spaces + loop)
            if len(perm) > 1:
                ret += self._inner_loops(perm[1:], sizes, indent, unrolls, bc_mod)
            else:
                ret += self._load_a(unrolls, indent+1)
                b_loads = self._load_b(unrolls, indent+1)
                maths = self._maths(unrolls, indent+1)
                b_take = (self.complex_complex and not self.have_bgp and not self.have_bgq) and 2 or 1
                m_take = unrolls['i']*(self.complex_complex and 2 or 1)
                while b_loads:
                    ret += b_loads[0:b_take]
                    ret += maths[0:m_take]
                    b_loads = b_loads[b_take:]
                    maths = maths[m_take:]
            ret.append(spaces + '}')

        if perm == ['k']:
            ret += self._store_c(unrolls, indent, bc_mod)

        return ret

    def gen(self, f, perm, size, func_name='mtxmq'):
        """Output generated code to file f

        Input:
            perm - an array of 'i', 'j', 'k' in the desired loop order
            size - { index : int, }

        Output:
            None
            Code printed to file f
        """

        if type(perm) is not list:
            perm = list(perm)

        if perm[-1] != 'k':
            raise Exception("k must be inner loop")

        lines = []

        # Header
        lines += self._header(func_name)

        # Temps Declaration
        lines += self._temp_dec(size)

        # Architecture Specific declarations, e.g. mask prep
        lines += self._extra()

        # Computation
        lines += self._inner_loops(perm, size)

        # Footer
        lines += self._footer()

        lines = self._post_process(lines)

        # Output
        for line in lines:
            print(line, file=f)


class MTXMAVX(MTXMGen):
    def __init__(self, *args):
        super().__init__(*args)
        self.vector_length = 4

        self.vector_type = '__m256d'
        self.vector_load = '_mm256_loadu_pd'
        self.vector_store = '_mm256_storeu_pd'
        self.vector_zero = '_mm256_setzero_pd()'

        self._mask = True
        self.mask_store = '_mm256_maskstore_pd'

        self.splat_type = '__m256d'
        self.splat_op = '_mm256_broadcast_sd'

        #self.complex_reverse_dup = '_mm256_permute_pd' # (_mm256_loadu_pd(addr), 5), could also use shuffle 5
        self.complex_dup = '_mm256_broadcast_pd'
        self.complex_dup_cast = '(const __m128d*)'
        #self.pair_splat = ''

    def _load_bz(self, spaces, addr, temp, k, j):
        return spaces + self._temp('_bz', k, j) + ' = _mm256_permute_pd(_mm256_broadcast_pd((const __m128d*){}),12);'.format(addr)

    def _load_br(self, spaces, addr, temp, k, j):
        return spaces + self._temp('_br', k, j) + ' = _mm256_permute_pd({}, 5);'.format(temp)

    def _fma(self, at, bt, ct):
        return ct + ' = _mm256_add_pd(_mm256_mul_pd(' + bt + ', ' + at + '), ' + ct + ');'

    def _fmaddsub(self, at, bt, ct):
        return ct + ' = _mm256_addsub_pd(' + ct + ', _mm256_mul_pd(' + at + ', ' + bt + '));'

    def _extra(self):
        if self.real_real:
            return [' ' * self.indent + """
    __m256i mask;
    j = effj % 4;
    switch (j) {
        case 0:
            mask = _mm256_set_epi32(-1,-1,-1,-1,-1,-1,-1,-1);
            break;
        case 1:
            mask = _mm256_set_epi32( 0, 0, 0, 0, 0, 0,-1,-1);
            break;
        case 2:
            mask = _mm256_set_epi32( 0, 0, 0, 0,-1,-1,-1,-1);
            break;
        case 3:
            mask = _mm256_set_epi32( 0, 0,-1,-1,-1,-1,-1,-1);
            break;
        default:
            return;
    }"""]
        else:
            return [' ' * self.indent + """
    __m256i mask;
    j = effj % 2;
    switch (j) {
        case 0:
            mask = _mm256_set_epi32(-1,-1,-1,-1,-1,-1,-1,-1);
            break;
        case 1:
            mask = _mm256_set_epi32( 0, 0, 0, 0,-1,-1,-1,-1);
            break;
        default:
            return;
    }"""]

class MTXMAVX2(MTXMAVX):
    def __init__(self, *args):
        super().__init__(*args)
        self.have_avx2 = True

    def _fma(self, at, bt, ct):
        return ct + ' = _mm256_fmadd_pd(' + ','.join([at,bt,ct]) + ');'

    def _fmaddsub(self, at, bt, ct):
        return ct + ' = _mm256_fmaddsub_pd(' + ','.join([at,bt,ct]) + ');'

class MTXMSSE(MTXMGen):
    def __init__(self, *args):
        super().__init__(*args)
        self.vector_length = 2

        self.vector_type = '__m128d'
        self.vector_load = '_mm_loadu_pd'
        self.vector_store = '_mm_storeu_pd'
        self.vector_zero = '_mm_setzero_pd()'

        self.splat_type = '__m128d'
        self.splat_op = '_mm_load1_pd'

        self.complex_reverse_dup = '_mm_loadr_pd' # aligned only!
        self.complex_dup = '_mm_loadu_pd'
        self.pair_splat = '_mm_load1_pd'

    def _fma(self, at, bt, ct):
        return ct + ' = _mm_add_pd(_mm_mul_pd(' + bt + ', ' + at + '), ' + ct + ');'

    def _fmaddsub(self, at, bt, ct):
        return "{2} = _mm_addsub_pd({2}, _mm_mul_pd({0}, {1}));".format(at, bt, ct)


class MTXMBGP(MTXMGen):
    def __init__(self, *args):
        super().__init__(*args)
        self.have_bgp = True
        self.vector_length = 2

        self.vector_type = '__complex__ double'
        self.vector_load = '__lfpd'
        self.vector_store = '__stfpd'
        self.vector_zero = '__cmplx(0.0,0.0)'

        self.splat_type = 'double'
        self.splat_op = '*'

        self.complex_reverse_dup = '__lfxd'
        self.complex_dup = '__lfpd'
        self.pair_splat = '*'

    def _fma(self, at, bt, ct):
        return ct + ' = __fxcpmadd(' + ct + ', ' + bt + ', ' + at + ');'

    def _fmaddsub(self, at, bt, ct):
        return ct + ' = __fxcxnpma(' + ct + ', ' + bt + ', ' + at + ');'

    def _post_process(self, lines):
        return [x.replace("__restrict__", "").replace("const", "").replace("double complex", "__complex__ double") for x in lines]

class MTXMBGQ(MTXMGen):
    def __init__(self, *args):
        super().__init__(*args)
        self.have_bgq = True
        self.vector_length = 4

        self.vector_type = 'vector4double'
        self.vector_load = 'vec_ld'
        self.vector_store = 'vec_st'
        self.vector_zero = '(vector4double)(0.0)'

        self.complex_dup = 'vec_ld2'

        self.splat_type = 'vector4double'
        self.splat_op = 'vec_lds'

    def _load_bz(self, spaces, addr, temp, k, j):
        t = self._temp('_bz', k, j)
        ret = spaces + t + ' = vec_ld2{};\n'.format(addr)
        ret += spaces + t + ' = vec_perm({0}, {0}, _cr_perm);'.format(t)
        return ret

    def _fma(self, at, bt, ct):
        if self.complex_complex:
            return ct + ' = vec_xmadd(' + at + ', ' + bt + ', ' + ct + ');'
        else:
            return ct + ' = vec_madd(' + at + ', ' + bt + ', ' + ct + ');'

    def _fmaddsub(self, at, bt, ct):
        return ct + ' = vec_xxnpmadd(' + bt + ', ' + at + ', ' + ct + ');'

    def _post_process(self, lines):
        return [x.replace("__restrict__", "").replace("const", "").replace("double complex", "__complex__ double") for x in lines]

    def _extra(self):
        if self.complex_real:
            return [' ' * self.indent + "vector4double _cr_perm = vec_gpci(0x9);"]
        else:
            return []
